package cz.frantovo.hrisniciSpameri.dao;

import cz.frantovo.hrisniciSpameri.dto.Cil;
import java.net.InetAddress;
import java.sql.Connection;
import java.sql.Date;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.Collection;
import java.util.logging.Level;

/**
 *
 * @author fiki
 */
public class GeneratorSouhrnuDAO extends HrisniciSuperDAO {

    private enum SQL {

        VLOZ_SOUHRN,
        SMAZ_SOUHRNY,
        IP_ADRESY_OBDOBI,
        IP_PREKLAD
    }

    /**
     * Vygeneruje souhrn pro daný cíl a datum a uloží ho do databáze
     * @param cil cíl, pro který generujeme souhrn
     * @param den den, pro který souhrn generujeme.
     */
    public void generujSouhrn(Cil cil, Date zacatek, Date konec) {
        Connection dbZdroj = getSpojeni(cil.getDatabaze());
        Connection dbSouhrn = getSpojeni(null);

        if (konec == null) {
            konec = zacatek;
        }

        if (dbZdroj == null || dbSouhrn == null) {
            /** TODO: Chyba spojení */
        } else {
            PreparedStatement ps = null;
            PreparedStatement psSmazat = null;
            ResultSet rs = null;
            try {
                /** pokud už něco vygenerovaného pro daný den a cíl máme, smažeme to */
                psSmazat = dbSouhrn.prepareStatement(getSQL(SQL.SMAZ_SOUHRNY));
                psSmazat.setInt(1, cil.getId());
                psSmazat.setDate(2, zacatek);
                psSmazat.setDate(3, konec);
                psSmazat.executeUpdate();

                ps = dbZdroj.prepareStatement(cil.getSelekt());
                ps.setDate(1, zacatek);
                ps.setDate(2, konec);
                rs = ps.executeQuery();
                PreparedStatement psSouhrn = dbSouhrn.prepareStatement(getSQL(SQL.VLOZ_SOUHRN));
                while (rs.next()) {
                    /**
                     * Souhrn bychom mohli vkládat pomocí SouhrnDAO,
                     * ale z výkonnostních důvodů to budeme dělat tady
                     * (jedno spojení a jeden předpřipravený příkaz).
                     */
                    psSouhrn.setDate(1, rs.getDate("den"));
                    psSouhrn.setString(2, rs.getString("ip"));
                    psSouhrn.setString(3, null);
                    psSouhrn.setInt(4, rs.getInt("pocet"));
                    psSouhrn.setInt(5, cil.getId());
                    psSouhrn.executeUpdate();
                }
                zavri(null, psSouhrn, null);

                /**
                 * Přeložíme všechny IP adresy daného období
                 * a aktualizujeme domény všude (i jiné cíle a jiná období).
                 */
                PreparedStatement psIPselekt = dbSouhrn.prepareStatement(getSQL(SQL.IP_ADRESY_OBDOBI));
                psIPselekt.setInt(1, cil.getId());
                psIPselekt.setDate(2, zacatek);
                psIPselekt.setDate(3, konec);
                PreparedStatement psIPupdate = dbSouhrn.prepareStatement(getSQL(SQL.IP_PREKLAD));
                ResultSet rsIP = psIPselekt.executeQuery();
                while (rsIP.next()) {
                    String ip = rsIP.getString("ip");
                    String domena = prelozIP(ip);
                    psIPupdate.setString(1, domena);
                    psIPupdate.setString(2, ip);
                    psIPupdate.executeUpdate();
                }
                zavri(null, psIPselekt, null);
                zavri(null, psIPupdate, null);

                /**
                 * Pokud se všechno povedlo, commitujeme, jinak nedochází k žádným změnám.
                 * Dané období se buď vygeneruje úspěšně, nebo vůbec.
                 * Přepokládáme vypnutý autocommit (to zajišťuje aplikační server).
                 */
                dbSouhrn.commit();
            } catch (Exception e) {
                log.log(Level.SEVERE, "Chyba při generování souhrnu.", e);
            } finally {
                zavri(dbZdroj, ps, rs);
                zavri(dbSouhrn, psSmazat, null);
            }
        }
    }

    /**
     * Vygeneruje denní souhrny pro všechny cíle za dané období.
     * @param zacatek začátek období – včetně
     * @param konec konec období – včetně
     */
    public void generujSouhrny(Date zacatek, Date konec) {
        Collection<Cil> cile = new CilDAO().getCile();
        for (Cil c : cile) {
            generujSouhrn(c, zacatek, konec);
        }
    }

    /**
     * Přeloží IP adresu na doménu
     * @param ip IP adresa
     * @return doména
     */
    private static String prelozIP(String ip) {
        try {
            return InetAddress.getByName(ip).getHostName();
        } catch (Exception e) {
            return "Chyba: nepodařilo se přeložit.";
        }
    }
}
